#include <avr/io.h>
#include "avrlibtypes.h"
#include "LM35_lib.h"
#include "timer.h"
#include "rprintf.h"

#define LM35_ADC_CHANNEL  0 
#define REFERENCE_VOLTAGE 5
#define DIRECTLY_PROPORTIONAL_FACTOR 5  // (0.2V ==> 1C) ==> (1.0V ==> 5C)




unsigned short getLM35ADCValue(){
		return (readADCChannel(LM35_ADC_CHANNEL));
}

unsigned short readADCChannel(uint8_t channel){

  uint8_t mux = channel;
  uint8_t i;
  uint16_t result;
 
  ADCSRA = (1<<ADEN) | (1<<ADPS2)| (1<<ADPS1) | (1<<ADPS0);    // Frequenzvorteiler 
                              					   // setzen auf 8 (1) und ADC aktivieren (1)
 
  ADMUX = mux;                      // Kanal waehlen
  ADMUX |= (0<<REFS1) | (1<<REFS0); // AVCC nutzen
 
  /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
     also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
  ADCSRA |= (1<<ADSC);              // eine ADC-Wandlung 
  while ( ADCSRA & (1<<ADSC) ) {
     ;     // auf Abschluss der Konvertierung warten 
  }
  result = ADCW;  // ADCW muss einmal gelesen werden,
                  // sonst wird Ergebnis der nchsten Wandlung
                  // nicht bernommen.
 
  /* Eigentliche Messung - Mittelwert aus 4 aufeinanderfolgenden Wandlungen */
  result = 0; 
  for( i=0; i<4; i++ )
  {
    ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
    while ( ADCSRA & (1<<ADSC) ) {
      ;   // auf Abschluss der Konvertierung warten
    }
    result += ADCW;		    // Wandlungsergebnisse aufaddieren
  }
  ADCSRA &= ~(1<<ADEN);             // ADC deaktivieren (2)
 
  result /= 4;                     // Summe durch vier teilen = arithm. Mittelwert
 
  return result;


}


float getLM35MeasuredVoltage(){

	unsigned short adcValue = getLM35ADCValue();
	
	float measuredVoltage = (adcValue * REFERENCE_VOLTAGE)/1024;

	return(measuredVoltage);
	 
}


float getLM35MeasuredTemperature(){

	float LM35MeasuredVoltage = getLM35MeasuredVoltage();

	float measuredTemperature = LM35MeasuredVoltage * DIRECTLY_PROPORTIONAL_FACTOR; 

	return(measuredTemperature);
	

}

